<?php

namespace FiloBlu\Newsletter\Helper;

class Import extends \FiloBlu\Newsletter\Helper\Data {

    protected $_subscriberFactory;
    protected $_subscriberInfoFactory;
    protected $_storeManager;
    protected $_customerInterface;

    public function __construct(
    \Magento\Framework\App\Helper\Context $context, \Magento\Store\Model\StoreManagerInterface $storeManager, \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory, \FiloBlu\Newsletter\Model\SubscriberInfoFactory $subscriberInfoFactory, \Magento\Customer\Api\CustomerRepositoryInterface $customerInterface
    ) {
        $this->_subscriberFactory = $subscriberFactory;
        $this->_subscriberInfoFactory = $subscriberInfoFactory;
        $this->_storeManager = $storeManager;
        $this->_customerInterface = $customerInterface;

        parent::__construct($context);
    }

    public function importDbData($data) {
        if ($data) {
            foreach ($data as $subscriberData) {
                
                // Magento default newsletter table
                $subscriber = $this->_subscriberFactory->create()->loadByEmail($subscriberData["email"]);
                $subscriber->setSubscriberEmail($subscriberData["email"]);

                $customerId = null;
                $customerStoreId = null;

                try {
                    $customer = $this->_customerInterface->get($subscriberData["email"]);
                    $customerId = $customer->getId();
                    $customerStoreId = $customer->getStoreId();
                } catch (\Exception $ex) {
                    // No customer found
                }

               
                $subscriber->setCustomerId($customerId);
                
                if ($customerStoreId) {
                    $subscriber->setStoreId($customerStoreId);
                } else {
                    $storeId = $this->_storeManager->getStores(false, true)[$subscriberData["store_code"]]->getId();
                    $subscriber->setStoreId($storeId);
                }

                $subscriber->setStatus($subscriberData["status"]);
                $subscriber->setSubscriberConfirmCode("IMPORT");


                $subscriberId = $subscriber->save()->getId();

                // Subscriber info table
                $subscriberInfo = $this->_subscriberInfoFactory->create()->load($subscriberId);
                $subscriberInfo->setSubscriberId($subscriberId);
                $subscriberInfo->addData($subscriberData);
                
                $subscriberInfo->save();
            }
        }
    }

    public function validateStoreCode($code) {
        return isset($this->_storeManager->getStores(false, true)[$code]);
    }
    
    public function validateStatus($status) {
        if ($status != (string)\Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED && $status != (string)\Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED) {
            return false;
        }
        else {
            return true;
        }
    }
}
