<?php

namespace FiloBlu\Newsletter\Model\Import;

use Magento\ImportExport\Model\Import\ErrorProcessing\ProcessingErrorAggregatorInterface;
use FiloBlu\Newsletter\Model\Import\Subscriber\RowValidatorInterface as ValidatorInterface;

class Subscriber extends \Magento\ImportExport\Model\Import\Entity\AbstractEntity {

    /**
     * Helper
     *
     * @var \FiloBlu\Newsletter\Helper\Import
     */
    protected $_helper;
    protected $logInHistory = true;
    protected $needColumnCheck = false;
    protected $_permanentAttributes = [
        "store_code",
        "email",
        "status"
    ];
    protected $validColumnNames = [
        "store_code",
        "email",
        "status"
    ];

    public function __construct(
    \Magento\Framework\Json\Helper\Data $jsonHelper, \Magento\ImportExport\Helper\Data $importExportData, \Magento\ImportExport\Model\ResourceModel\Import\Data $importData, \Magento\Framework\App\ResourceConnection $resource, \Magento\ImportExport\Model\ResourceModel\Helper $resourceHelper, ProcessingErrorAggregatorInterface $errorAggregator, \FiloBlu\Newsletter\Helper\Import $helper
    ) {
        $this->jsonHelper = $jsonHelper;
        $this->_importExportData = $importExportData;
        $this->_resourceHelper = $resourceHelper;
        $this->_dataSourceModel = $importData;
        $this->_resource = $resource;
        $this->_connection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $this->errorAggregator = $errorAggregator;
        $this->_helper = $helper;

        $this->getAllowedColumns();
    }

    protected function getAllowedColumns() {

        $fieldsMapping = unserialize($this->_helper->getConfig('general/fields_mapping'));

        if ($fieldsMapping && count($fieldsMapping)) {

            foreach ($fieldsMapping as $field) {
                $this->validColumnNames[] = $field["dbcolumnname"];
            }
        }
    }

    /**
     * Create Advanced price data from raw data.
     *
     * @throws \Exception
     * @return bool Result of operation.
     */
    protected function _importData() {

        if (\Magento\ImportExport\Model\Import::BEHAVIOR_REPLACE == $this->getBehavior()) {
            $this->updateData();
        }

        return true;
    }

    protected function updateData() {
        $i = 0;

        while ($bunch = $this->_dataSourceModel->getNextBunch()) {

            $entityList = [];
            foreach ($bunch as $rowNum => $rowData) {

                if ($this->getErrorAggregator()->hasToBeTerminated()) {
                    $this->getErrorAggregator()->addRowToSkip($rowNum);
                    continue;
                }

                $i++;

                $entityList[$i]["store_code"] = $rowData["store_code"];
                $entityList[$i]["email"] = $rowData["email"];
                $entityList[$i]["status"] = $rowData["status"];


                foreach ($this->validColumnNames as $column) {
                    if (isset($rowData[$column])) {
                        $entityList[$i][$column] = $rowData[$column];
                    }
                }
            }

            $this->countItemsCreated += $i;

            $this->_helper->importDbData($entityList);
        }

        return $this;
    }

    /**
     * Entity type code getter.
     *
     * @return string
     */
    public function getEntityTypeCode() {
        return 'newsletter_subscriber_import';
    }

    public function validateRow(array $rowData, $rowNum) {
        if (\Magento\ImportExport\Model\Import::BEHAVIOR_REPLACE == $this->getBehavior()) {

            // Check store_code
            if (!isset($rowData["store_code"]) || empty($rowData["store_code"])) {
                $this->addRowError(ValidatorInterface::ERROR_STORE_CODE_IS_EMPTY, $rowNum);
            } else {
                if (!$this->_helper->validateStoreCode($rowData["store_code"])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_STORE . " " . $rowData["store_code"], $rowNum);
                }
            }

            // Check email
            if (!isset($rowData["email"]) || empty($rowData["email"])) {
                $this->addRowError(ValidatorInterface::ERROR_EMAIL_IS_EMPTY, $rowNum);
            } else {
                if (!\Zend_Validate::is($rowData["email"], 'EmailAddress')) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_EMAIL . " " . $rowData["email"], $rowNum);
                }
            }
            
            // Check status
            if (!isset($rowData["status"]) || empty($rowData["status"])) {
                $this->addRowError(ValidatorInterface::ERROR_STATUS_IS_EMPTY, $rowNum);
            } else {
                if (!$this->_helper->validateStatus($rowData["status"])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_STATUS . " - currentStatus: " . $rowData["status"], $rowNum);
                }
            }
        }

        return !$this->getErrorAggregator()->isRowInvalid($rowNum);
    }

}
